;==========================================================================
; AsmIOLib.asm
; twl 6 apr 12
; this is a library of functions to support basic i/o in asm programs
;
;==========================================================================
		.386P
	    .model  flat
		extern  _ExitProcess@4:near		;api call to exit program
		extern  _GetStdHandle@4:near	;api call to get std handle
		extern  _WriteConsoleA@20:near	;api call to write to the console
		extern	_ReadConsoleA@20:near	;api call to read from the console
;
; make these routines available to other asm programs
;			
		public _output
		public _input
		public _exit
;--------------------------------------------------------------------------			
	    .data						;start of the DATA segment
;--------------------------------------------------------------------------
; constants
;	        
STD_INPUT_HANDLE	equ	-10			;api uses -10 to mean the input handle
STD_OUTPUT_HANDLE	equ	-11			;api uses -11 to mean the output handle
;
; memory storage
;
outHandle   DWORD 0					;stores the output handle
inHandle    DWORD 0					;stores the input handle
buffer      BYTE 80 dup(?)			;create an input buffer
bytesRead   DWORD ?					;holds number of bytes entered by the user
bytesWritten DWORD ?				;holds number of bytes written

;--------------------------------------------------------------------------
		.code						;start of the code segment
;--------------------------------------------------------------------------
;output -	prints text messages to the console screen
;			Calling routing needs to put the number of characters to print into the
;			EAX register, and the address of the message in the EBX register. This
;			routine uses the Win32 API funciton _WriteConsoleA@20

_output	PROC								

		;save the caller's regs
		push eax					;holds the number of characters to print
		push ebx					;holds the address of the message to print
		push ecx					;use internally here
		mov ecx, eax				;put the number of characters in cx
		;
		; we need to get handles
		;
        push    STD_OUTPUT_HANDLE	;tell the function what type of handle
        call    _GetStdHandle@4		;call the api function
        mov     outHandle, eax		;store the handle for our use

		; WriteConsole(handle, &msg[0], 13, &written, 0) 
		; Parameters are pushed onto the stack in reverse order from 
		; how they are listed
        ;
        push    0					;last parameter first - reserved by MS
        push    offset bytesWritten	;tell function where to store the bytes written
        push    ecx					;tell function how many chars to write
        push    ebx					;tell function where the message is to write
        push    outHandle			;tell function what handle to use
        call    _WriteConsoleA@20	;call the function
        
        ;restore the caller's regs
        pop eax
        pop ebx
        pop ecx
        
        ret							;return to the caller
        
_output	ENDP		

;--------------------------------------------------------------------------
;input -	Reads characters typed on the console keyboard. No registers need to be
;			passed. This routine uses the Win32 API funciton _ReadConsoleA@20. The
;			number of characters typed is stored in the bytesRead buffer
;

_input	PROC								
		;
		; we need to get the input handle
		;
		push    STD_INPUT_HANDLE	;tell the function what type of handle
        call    _GetStdHandle@4		;call the api function
        mov     inHandle, eax		;store the handle for our use

		;
		; ReadConsole(handle, &inputbuffer, numberToRead, 0)
        ;
		; parameters are pushed on the stack in reverse order
		
        push    0					;last parameter first - reserved by MS
        push    offset bytesRead	;tell function where to store the bytes actually read
        push    80					;tell function max chars to read
        push    offset buffer		;tell function where to store user's input
        push    inHandle			;tell function what handle to use
        call    _ReadConsoleA@20	;call the function
        mov eax, offset buffer		;pass the buffer address back to the caller in eax
        mov ecx, bytesRead			;pass the number of bytes read back to the caller in ecx
        
        ret							;return to user
_input	ENDP
;--------------------------------------------------------------------------
; exit

_exit	PROC

        push    0
        call    _ExitProcess@4
        
_exit	ENDP
;--------------------------------------------------------------------------
		END							;end of the source code
;--------------------------------------------------------------------------